use anyhow::Result;
use async_graphql::SimpleObject;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};

#[derive(SimpleObject, FromRow, Deserialize, Serialize)]
pub struct Typ {
    /// Die Datenbank-ID
    pub id: i32,

    /// Der Name eines Typs
    name: String,
}

impl Typ {
    pub async fn create(pool: &PgPool, name: &str) -> Result<Typ> {
        let row = sqlx::query!("INSERT INTO typen(name) VALUES ($1) RETURNING id", name)
            .fetch_one(pool)
            .await?;

        Ok(Typ {
            id: row.id,
            name: name.to_string(),
        })
    }

    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Typ> {
        let row = sqlx::query_as!(Typ, "SELECT * FROM typen WHERE id = $1", id)
            .fetch_one(pool)
            .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Typ>> {
        let rows = sqlx::query_as!(Typ, "SELECT id, name FROM typen")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn update(pool: &PgPool, id: &i32, name: &str) -> Result<Typ> {
        sqlx::query!("UPDATE typen SET name=$1 WHERE id = $2", name, id)
            .execute(pool)
            .await?;

        let t = Typ::read_one(pool, id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<()> {
        sqlx::query!("DELETE FROM typen WHERE id = $1", id)
            .execute(pool)
            .await?;

        Ok(())
    }
}
